@@ -1,5 +1,5 @@
 """Traitlets Python configuration system"""
-from warnings import warn
+import typing as _t
 
 from . import traitlets
 from ._version import __version__, version_info
@@ -7,6 +7,7 @@ from .traitlets import *
 from .utils.bunch import Bunch
 from .utils.decorators import signature_has_traits
 from .utils.importstring import import_item
+from .utils.warnings import warn
 
 __all__ = [
     "traitlets",
@@ -20,7 +21,7 @@ __all__ = [
 
 
 class Sentinel(traitlets.Sentinel):  # type:ignore[name-defined]
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: _t.Any, **kwargs: _t.Any) -> None:
         super().__init__(*args, **kwargs)
         warn(
             """
